package com.freiheit.hibernate;

import java.sql.*;
import java.util.*;

public class HsqldbManager {
    Connection _connection;
    
    public HsqldbManager() throws ClassNotFoundException, SQLException {
        Class.forName( "org.hsqldb.jdbcDriver" );
        _connection = DriverManager.getConnection( "jdbc:hsqldb:.", "sa", "" );    
    }

    public void createTestTables() throws SQLException {
        query( "CREATE TABLE customer ( id INT IDENTITY PRIMARY KEY, firstname VARCHAR NOT NULL, lastname VARCHAR NOT NULL, username VARCHAR NOT NULL, password VARCHAR NOT NULL, active BIT DEFAULT 0 NOT NULL, creation_date DATE NOT NULL, changing_date DATE );" );

        query( "CREATE TABLE country ( id INT IDENTITY PRIMARY KEY, name VARCHAR NOT NULL, iso_code VARCHAR NOT NULL );" );

        query( "CREATE TABLE address ( id INT IDENTITY PRIMARY KEY, customer_id INT NOT NULL, country_id INT NOT NULL, firstname VARCHAR NOT NULL, lastname VARCHAR NOT NULL, street VARCHAR NOT NULL, number VARCHAR NOT NULL, postal_code VARCHAR NOT NULL, city VARCHAR NOT NULL, creation_date DATE NOT NULL, changing_date DATE, FOREIGN KEY( customer_id ) REFERENCES customer ( id ), FOREIGN KEY( country_id ) REFERENCES country ( id ) );" );

        query( "CREATE TABLE creditcard_company ( id INT IDENTITY PRIMARY KEY, name VARCHAR NOT NULL);" );
        
        query( "CREATE TABLE bank ( id INT IDENTITY PRIMARY KEY, name VARCHAR NOT NULL, bank_code VARCHAR NOT NULL );" );

        query( "CREATE TABLE payment_method ( id INT IDENTITY PRIMARY KEY, customer_id INT NOT NULL, creditcard_company_id INT, bank_id INT, type INT NOT NULL, owner VARCHAR NOT NULL, number VARCHAR NOT NULL, valid_month INT, valid_year INT, creation_date DATE NOT NULL, changing_date DATE, FOREIGN KEY( customer_id ) REFERENCES customer( id ), FOREIGN KEY( creditcard_company_id ) REFERENCES creditcard_company( id ), FOREIGN KEY( bank_id ) REFERENCES bank( id ) );" );

        query( "CREATE TABLE keyword ( id INT IDENTITY PRIMARY KEY, name VARCHAR NOT NULL, creation_date DATE NOT NULL, changing_date DATE );" );

        query( "CREATE TABLE keyword_customer_relation ( id INT IDENTITY PRIMARY KEY, customer_id INT NOT NULL, keyword_id INT NOT NULL, FOREIGN KEY( customer_id ) REFERENCES customer( id ), FOREIGN KEY( keyword_id ) REFERENCES keyword( id ) );" );
    }
    
    public void addMasterData() throws SQLException {
        update( "" );
        update( "INSERT INTO country( name, iso_code ) VALUES ( 'Deutschland', 'DE' );" );
        update( "INSERT INTO country( name, iso_code ) VALUES ( 'Burkina Faso', 'BF' );" );
        update( "INSERT INTO country( name, iso_code ) VALUES ( 'Bahamas', 'BS' );" );
        
        update( "INSERT INTO creditcard_company( name ) VALUES ( 'Mastercard' );" );
        update( "INSERT INTO creditcard_company( name ) VALUES ( 'American Express' );" );
        update( "INSERT INTO creditcard_company( name ) VALUES ( 'Eurocard' );" );
        update( "INSERT INTO creditcard_company( name ) VALUES ( 'Visa Card' );" );

        update( "INSERT INTO bank( name, bank_code ) VALUES ( 'Raiffbk Bevern, Kr Holzminden', '25069278' );" );
        update( "INSERT INTO bank( name, bank_code ) VALUES ( 'BBk Osnabrck', '26500000' );" );
        update( "INSERT INTO bank( name, bank_code ) VALUES ( 'Dresdner Bank Einbeck', '26281420' );" );
        update( "INSERT INTO bank( name, bank_code ) VALUES ( 'Deutsche Bank Hildesheim', '25970074' );" );
        update( "INSERT INTO bank( name, bank_code ) VALUES ( 'Bankhaus Fortmann Oldenburg (Oldb)', '28030300' );" );
        update( "INSERT INTO bank( name, bank_code ) VALUES ( 'BBk Dsseldorf', '30000000' );" );
        update( "INSERT INTO bank( name, bank_code ) VALUES ( 'Commerzbank Bad Bramstedt', '21241540' );" );
    }
    
    public void query( String expression ) throws SQLException {
        Statement statement = null;

        statement = _connection.createStatement();
        statement.executeQuery( expression );

        statement.close();
    }

    public void update( String expression ) throws SQLException {
        Statement statement = null;

        statement = _connection.createStatement();
        statement.executeUpdate( expression );

        statement.close();
    }
    
    private String dumpTableByName( String table ) throws SQLException {
        Statement statement = null;
        StringBuffer buffer = new StringBuffer();
        
        statement = _connection.createStatement();
        dumpResultSet( statement.executeQuery( "select * from " + table + ";" ), buffer );
        statement.close();

        return buffer.toString();
    }

    public String dumpAddressTable() throws SQLException {
        return dumpTableByName( "ADDRESS" );
    }
    
    public String dumpBankTable() throws SQLException {
        return dumpTableByName( "BANK" );
    }
    
    public String dumpCountryTable() throws SQLException {
        return dumpTableByName( "COUNTRY" );
    }
    
    public String dumpCreditCardCompanyTable() throws SQLException {
        return dumpTableByName( "CREDITCARD_COMPANY" );
    }
    
    public String dumpCustomerTable() throws SQLException {
        return dumpTableByName( "CUSTOMER" );
    }
    
    public String dumpKeywordTable() throws SQLException {
        return dumpTableByName( "KEYWORD" );
    }
    
    public String dumpKeywordCustomerRelationTable() throws SQLException {
        return dumpTableByName( "KEYWORD_CUSTOMER_RELATION" );
    }
    
    public String dumpPaymentMethodTable() throws SQLException {
        return dumpTableByName( "PAYMENT_METHOD" );
    }
    
    public String dumpAllTables() throws SQLException {
        StringBuffer buffer;

        buffer = new StringBuffer();

        buffer.append( dumpAddressTable() );
        buffer.append( dumpBankTable() );
        buffer.append( dumpCountryTable() );
        buffer.append( dumpCreditCardCompanyTable() );
        buffer.append( dumpCustomerTable() );
        buffer.append( dumpKeywordTable() );
        buffer.append( dumpKeywordCustomerRelationTable() );
        buffer.append( dumpPaymentMethodTable() );

        return buffer.toString();
    }

    public void dumpResultSet( ResultSet resultSet, StringBuffer buffer ) throws SQLException {
        ResultSetMetaData metaData;
        int columnCount;
        int columnSize[];
        String columnValue;
        List columnValues, columnValueList;
        StringBuffer separatorRow;
        
        metaData = resultSet.getMetaData();
        columnCount = metaData.getColumnCount();
        
        columnSize = new int[ columnCount ];
        columnValueList = new LinkedList();
        
        columnValues = new LinkedList();
        for ( int i = 1; i <= columnCount; i++ ) {
            columnValues.add( metaData.getColumnName( i ) );
            columnSize[ i - 1 ] = metaData.getColumnName( i ).length();
        }
        columnValueList.add( columnValues );

        while ( resultSet.next() ) {
            columnValues = new LinkedList();
            for ( int i = 1; i <= columnCount; i++ ) {
                if ( resultSet.getObject( i ) != null ) {
                    columnValue = resultSet.getObject( i ).toString();
                } else {
                    columnValue = "[null]";
                }
                columnValues.add( columnValue );
                if ( columnValue.length() > columnSize[ i - 1 ] ) {
                    columnSize[ i -1 ] = columnValue.length();
                }
            }
            columnValueList.add( columnValues );
        }

        if ( columnValueList.size() > 1 ) { 
            separatorRow = new StringBuffer( "+" );
            for ( int i = 0; i < columnCount; i++ ) {
                separatorRow.append( "-" );
                separatorRow.append( fillUp( columnSize[ i ], '-' ) );
                separatorRow.append( "-+" );
            }
            separatorRow.append( "\n" );

            buffer.append( metaData.getTableName( 1 ) ).append( ":\n" );
            buffer.append( separatorRow );

            for ( int i = 0; i < columnValueList.size(); i++ ) {
                columnValues = (List)columnValueList.get( i );
                buffer.append( "|" );
                for ( int j = 0; j < columnCount; j++ ) {
                    buffer.append( " " );
                    columnValue = (String)columnValues.get( j );
                    buffer.append( columnValue );
                    buffer.append( fillUp( columnSize[ j ] - columnValue.length(), ' ' ) );
                    buffer.append( " |" );    
                }
                buffer.append( "\n" );

                if ( i == 0 ) {
                    buffer.append( separatorRow );
                }
            }
            buffer.append( separatorRow );
        } else {
            buffer.append( metaData.getTableName( 1 ) ).append( ": empty table\n" );
        }
    }
    
    private String fillUp( int amount, char fillChar ) {
        StringBuffer result;

        result = new StringBuffer();
        for ( int i = 0; i < amount; i++ ) {
            result.append( fillChar );
        }

        return result.toString();
    }
    
    public Connection getConnection() {
        return _connection;
    }

    public void shutdown() throws SQLException {
        _connection.close();
    }
}
