package com.freiheit.hibernate.beans;

import net.sf.hibernate.CallbackException;
import net.sf.hibernate.Lifecycle;
import net.sf.hibernate.Session;

import java.io.Serializable;

import java.util.Calendar;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;

public class Address implements Lifecycle, Serializable {

    private Integer _id;
    private String _firstname;
    private String _lastname;
    private String _street;
    private String _number;
    private String _postalCode;
    private String _city;
    private Calendar _creationDate;
    private Calendar _changingDate;
    private Country _country;
    private Customer _customer;

    public Address() {
    }

    // ---- Lifecycle methods --------------------------------------------------

    public boolean onSave(Session s) throws CallbackException {
        setCreationDate( Calendar.getInstance() );
        setChangingDate( Calendar.getInstance() );
        return false; // allow save
    }

    public boolean onUpdate(Session s) throws CallbackException {
        setChangingDate( Calendar.getInstance() );
        return false; // allow update
    }

    public boolean onDelete(Session s) throws CallbackException {
        return false; // allow delete
    }

    public void onLoad(Session s, Serializable id) {}

    // ---- general methods ----------------------------------------------------

    public String getDisplayName() {
        StringBuffer displayName;

        displayName = new StringBuffer();
        displayName.append( _firstname );
        displayName.append( " " );
        displayName.append( _lastname );
        displayName.append( ", " );
        displayName.append( _street );
        displayName.append( " " );
        displayName.append( _number );

        return displayName.toString();
    }

    // ---- Bean methods -------------------------------------------------------

    public Integer getId() {
        return _id;
    }
    public void setId(Integer id) {
        _id = id;
    }

    public String getFirstname() {
        return _firstname;
    }
    public void setFirstname(String firstname) {
        _firstname = firstname;
    }

    public String getLastname() {
        return _lastname;
    }
    public void setLastname(String lastname) {
        _lastname = lastname;
    }

    public String getStreet() {
        return _street;
    }
    public void setStreet(String street) {
        _street = street;
    }

    public String getNumber() {
        return _number;
    }
    public void setNumber(String number) {
        _number = number;
    }

    public String getPostalCode() {
        return _postalCode;
    }
    public void setPostalCode(String postalCode) {
        _postalCode = postalCode;
    }

    public String getCity() {
        return _city;
    }
    public void setCity(String city) {
        _city = city;
    }

    public Calendar getCreationDate() {
        return _creationDate;
    }
    public void setCreationDate(Calendar creationDate) {
        _creationDate = creationDate;
    }

    public Calendar getChangingDate() {
        return _changingDate;
    }
    public void setChangingDate(Calendar changingDate) {
        _changingDate = changingDate;
    }

    public Country getCountry() {
        return _country;
    }
    public void setCountry(Country country) {
        _country = country;
    }

    public void setCustomer( Customer customer ) {
        _customer = customer;
    }

    public Customer getCustomer() {
        return _customer;
    }

    // ---- standard object methods --------------------------------------------

    public String toString() {
        return ToStringBuilder.reflectionToString( this, ToStringStyle.SIMPLE_STYLE );
    }

    public boolean equals(Object other) {
        if ( !(other instanceof Address) ) return false;
        Address castOther = (Address) other;
        return new EqualsBuilder()
            .append(_id, castOther._id)
            .isEquals();
    }

    public int hashCode() {
        return new HashCodeBuilder()
            .append(_id)
            .toHashCode();
    }

}
