package com.freiheit.hibernate.beans;

import net.sf.hibernate.CallbackException;
import net.sf.hibernate.Lifecycle;
import net.sf.hibernate.Session;

import java.io.Serializable;

import java.util.Calendar;
import java.util.HashSet;
import java.util.Set;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;


public class Customer implements Lifecycle, Serializable {

    private Integer _id;

    private String _firstname;
    private String _lastname;
    private String _username;
    private String _password;
    private boolean _active;
    private Calendar _creationDate;
    private Calendar _changingDate;
    private Set _addresses;
    private Set _bankAccounts;
    private Set _creditCards;
    private Set _keywords;

    public Customer() {
    }

    // ---- Lifecycle methods --------------------------------------------------

    public boolean onSave(Session s) throws CallbackException {
        setCreationDate( Calendar.getInstance() );
        setChangingDate( Calendar.getInstance() );
        return false; // allow save
    }

    public boolean onUpdate(Session s) throws CallbackException {
        setChangingDate( Calendar.getInstance() );
        return false; // allow update
    }

    public boolean onDelete(Session s) throws CallbackException {
        return false; // allow delete
    }

    public void onLoad(Session s, Serializable id) {}


    // ---- Bean methods -------------------------------------------------------

    public Integer getId() {
        return _id;
    }
    public void setId(Integer id) {
        _id = id;
    }

    public String getFirstname() {
        return _firstname;
    }
    public void setFirstname(String firstname) {
        _firstname = firstname;
    }

    public String getLastname() {
        return _lastname;
    }
    public void setLastname(String lastname) {
        _lastname = lastname;
    }

    public String getUsername() {
        return _username;
    }
    public void setUsername(String username) {
        _username = username;
    }

    public String getPassword() {
        return _password;
    }
    public void setPassword(String password) {
        _password = password;
    }

    public boolean isActive() {
        return _active;
    }
    public void setActive(boolean active) {
        _active = active;
    }

    public Calendar getCreationDate() {
        return _creationDate;
    }
    public void setCreationDate(Calendar creationDate) {
        _creationDate = creationDate;
    }

    public Calendar getChangingDate() {
        return _changingDate;
    }
    public void setChangingDate(Calendar changingDate) {
        _changingDate = changingDate;
    }

    public void setAddresses( Set addresses ) {
        _addresses = addresses;
    }

    public Set getAddresses() {
        return _addresses;
    }

    public void setBankAccounts( Set bankAccounts ) {
        _bankAccounts = bankAccounts;
    }

    public Set getBankAccounts() {
        return _bankAccounts;
    }

    public void setCreditCards( Set creditCards ) {
        _creditCards = creditCards;
    }

    public Set getCreditCards() {
        return _creditCards;
    }

    public void setKeywords( Set keywords ) {
        _keywords = keywords;
    }

    public Set getKeywords() {
        return _keywords;
    }
    
    // ---- helper methods for the sets ----------------------------------------
    public void addAddress( Address address ) {
        address.setCustomer( this );
        if ( _addresses == null ) {
            _addresses = new HashSet();
        }
        _addresses.add( address );
    }
    
    public void addBankAccount( BankAccount bankAccount ) {
        bankAccount.setCustomer( this );
        if ( _bankAccounts == null ) {
            _bankAccounts = new HashSet();
        }
        _bankAccounts.add( bankAccount );
    }
    
    public void addCreditCard( CreditCard creditCard ) {
        creditCard.setCustomer( this );
        if ( _creditCards == null ) {
            _creditCards = new HashSet();
        }
        _creditCards.add( creditCard );
    }
    
    public void addKeyword( Keyword keyword ) {
        if ( _keywords == null ) {
            _keywords = new HashSet();
        }
        _keywords.add( keyword );
    }
    
    // ---- standard object methods --------------------------------------------

    public String toString() {
        StringBuffer buffer;

        buffer = new StringBuffer( "[" ).append( _id ).append( "] " );
        buffer.append( _lastname ).append( ", " ).append( _firstname );
        buffer.append( " ( " ).append( _username ).append( " / " ).append( _password ).append( " )" );
        //buffer.append( _addresses );
        //buffer.append( _bankAccounts );
        //buffer.append( _creditCards );
        //buffer.append( _keywords );
        
        return buffer.toString();
    }

    public boolean equals(Object other) {
        if ( !(other instanceof Customer) ) return false;
        Customer castOther = (Customer) other;
        return new EqualsBuilder()
            .append(_id, castOther._id)
            .isEquals();
    }

    public int hashCode() {
        return new HashCodeBuilder()
            .append(_id)
            .toHashCode();
    }

}
