package com.freiheit.hibernate.beans;

import net.sf.hibernate.CallbackException;
import net.sf.hibernate.Lifecycle;
import net.sf.hibernate.Session;

import java.io.Serializable;
import java.util.Calendar;
import java.util.HashSet;
import java.util.Set;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;

public class Keyword implements Lifecycle, Serializable {
    private Integer _id;
    private String _name;
    private Calendar _creationDate;
    private Calendar _changingDate;
    private Set _customers;

    public Keyword() {
        this( "" );
    }

    public Keyword( String name ) {
        _name = name;
        _customers = new HashSet();
    }
    
    // ---- Lifecycle methods --------------------------------------------------

    public boolean onSave(Session s) throws CallbackException {
        setCreationDate( Calendar.getInstance() );
        setChangingDate( Calendar.getInstance() );
        return false; // allow save
    }

    public boolean onUpdate(Session s) throws CallbackException {
        setChangingDate( Calendar.getInstance() );
        return false; // allow update
    }

    public boolean onDelete(Session s) throws CallbackException {
        return false; // allow delete
    }

    public void onLoad(Session s, Serializable id) {}
    
    // ---- bean methods -------------------------------------------------------

    public Integer getId() {
        return _id;
    }
    public void setId( Integer id ) {
        _id = id;
    }

    public String getName() {
        return _name;
    }
    public void setName( String name ) {
        _name = name;
    }

    public void setCreationDate( Calendar creationDate ) {
        _creationDate = creationDate;
    }

    public Calendar getCreationDate() {
        return _creationDate;
    }

    public void setChangingDate( Calendar changingDate ) {
        _changingDate = changingDate;
    }

    public Calendar getChangingDate() {
        return _changingDate;
    }

    public void setCustomers( Set customers ) {
        _customers = customers;
    }

    public Set getCustomers() {
        return _customers;
    }

    // --- standard object methods ---------------------------------------------

    public String toString() {
        return ToStringBuilder.reflectionToString( this, ToStringStyle.SIMPLE_STYLE );
    }

    public boolean equals(Object other) {
        if ( !(other instanceof Keyword) ) return false;
        Keyword castOther = (Keyword) other;
        if ( _id != null ) {
        return new EqualsBuilder()
            .append(_id, castOther._id)
            .isEquals();
        } else {
            return false;
        }
    }
/*
    public int hashCode() {
        return new HashCodeBuilder()
            .append(_id)
            .toHashCode();
    }
*/
}
