package com.freiheit.hibernate.beans;

import net.sf.hibernate.CallbackException;
import net.sf.hibernate.Lifecycle;
import net.sf.hibernate.Session;

import java.io.Serializable;

import java.util.Calendar;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;


abstract public class PaymentMethod implements Lifecycle, Serializable {

    private Long _id;
    private String _number;
    private String _owner;
    private Calendar _creationDate;
    private Calendar _changingDate;
    private Customer _customer;

    // ---- Lifecycle methods --------------------------------------------------

    public boolean onSave(Session s) throws CallbackException {
        setCreationDate( Calendar.getInstance() );
        setChangingDate( Calendar.getInstance() );
        return false; // allow save
    }

    public boolean onUpdate(Session s) throws CallbackException {
        setChangingDate( Calendar.getInstance() );
        return false; // allow update
    }

    public boolean onDelete(Session s) throws CallbackException {
        return false; // allow delete
    }

    public void onLoad(Session s, Serializable id) {}

    // ---- general methods ----------------------------------------------------

    public String getDisplayNumber() {
        int length = _number.length();
        return _number.substring( length - 4, length );
    }

    // ---- Bean methods -------------------------------------------------------

    public void setId( Long id ) {
        _id = id;
    }
    public Long getId() {
        return _id;
    }

    public void setNumber( String number ) {
        _number = number;
    }
    public String getNumber() {
        return _number;
    }

    public void setOwner( String owner ) {
        _owner = owner;
    }
    public String getOwner() {
        return _owner;
    }

    public Calendar getCreationDate() {
        return _creationDate;
    }
    public void setCreationDate(Calendar creationDate) {
        _creationDate = creationDate;
    }

    public Calendar getChangingDate() {
        return _changingDate;
    }
    public void setChangingDate(Calendar changingDate) {
        _changingDate = changingDate;
    }

    public void setCustomer( Customer customer ) {
        _customer = customer;
    }
    public Customer getCustomer() {
        return _customer;
    }

    // ---- standard object methods --------------------------------------------

    public String toString() {
        return ToStringBuilder.reflectionToString( this, ToStringStyle.SIMPLE_STYLE );
    }

    public boolean equals(Object other) {
        if ( !(other instanceof PaymentMethod) ) return false;
        PaymentMethod castOther = (PaymentMethod) other;
        return new EqualsBuilder()
            .append(_id, castOther._id)
            .isEquals();
    }

    public int hashCode() {
        return new HashCodeBuilder()
            .append(_id)
            .toHashCode();
    }
}

/* Emacs:
 * Local variables:
 * c-basic-offset: 4
 * tab-width: 4
 * indent-tabs-mode: nil
 * compile-command: "ant -emacs -find build.xml"
 * End:
 * vi:set tabstop=4 shiftwidth=4 nowrap:
 */
